 // Maximum-likelihood fit when p=f(1988) or p is release specific
 // K.B. Newman, Final version 12 January 2003; 
 //   --- original by Dave Fournier

 // Logit links for S and p
 // If making a run with release-specific p's, then comment out
 //   the init_vector betap(1,2) and pre_p and mu_p lines
 //   and remove comments from init_bounded_vector pn and mu_p lines

DATA_SECTION
  init_int nu               // the number of upstream releases
  init_int nd               // the number of downstream releases
  init_int nc               // the number of covariates
  init_ivector i_j(1,nu)    // the downstream release associated with each
                            // upstream release
  init_vector Ru(1,nu)      // the size of each upstream release
  init_vector Rd(1,nd)      // the size of each downstream release
  init_matrix yu(1,nu,1,2)  // the river and ocean catches of each 
                            // upstream release 
  init_vector yo(1,nd)      // the ocean catches of each 
                            // downstream release 
  matrix tyu(1,2,1,nu)      // transpose of the river and ocean catches of each 
                            // upstream release 
  init_vector year88(1,nu)  // indicator for 1988
  init_matrix covariates(1,nu,1,nc) // the covariates
 !! tyu=trans(yu);

PARAMETER_SECTION
  init_number spar0                             // coefficients for survival covariates
  init_vector spar1(2,nc,2)                     // coefficients for survival covariates
  init_bounded_vector pn(1,nu,1.e-6,1)          // net capture rates
  // init_vector betap(1,2)                        // params for capture rate of the net for each upstream release
  init_bounded_vector po(1,nd,1.e-6,1)          // ocean capture rate for each downstream release 
  vector scoff(1,nc);
  objective_function_value ML

INITIALIZATION_SECTION
  spar0 -0.1

PROCEDURE_SECTION

  scoff.initialize();
  scoff(1)=spar0;
  if (active(spar1)) scoff(2,nc)=spar1;

  // ------- Expected value calculations
  dvar_vector pre_surv=mfexp(covariates*scoff);   
  dvar_vector mu_s   = elem_div(pre_surv,    1.0+pre_surv);     // logistic link function

  // if p=f(covariates)
  // dvar_vector pre_p  = exp(betap(1)+betap(2)*year88);              
  // dvar_vector mu_p   = elem_div(pre_p,     1.0+pre_p);          // logistic link function

  // if not ...
  dvar_vector mu_p = pn;

  dvar_vector mu_piu = po(i_j);
  dvar_vector mu_sp  = elem_prod(mu_s,mu_p);
  dvar_vector mu_spnpiu = elem_prod(elem_prod(mu_s,1-mu_p),mu_piu);

  // *********************************************************************************************************//
  ML  =  sum(elem_prod(tyu[1],log(mu_sp)))+sum(elem_prod(tyu[2],log(mu_spnpiu))) +
         sum(elem_prod(Ru-tyu[1]-tyu[2],log(1-mu_sp-mu_spnpiu)));
  ML  += sum(elem_prod(yo,log(po))) + sum(elem_prod(Rd-yo,log(1-po)));
  ML  += 0.001*norm2(scoff);                           // little penalty for stabilizing
  ML  = - ML;                                          // change sign for minimization
