 // Pseudo-likelihood fit when p=f(1988) or p=release specific
 // K. Newman, 12 January 2003 Final version; based on pgm by
 //    Dave Fournier

 // Logit links for S and p
 // If making a run with release-specific p's, then comment out
 //   the init_vector betap(1,2) and pre_p and mu_p lines
 //   and remove comments from init_bounded_vector pn and mu_p lines

DATA_SECTION
  init_int nu                       // the number of upstream releases
  init_int nd                       // the number of downstream releases
  init_int nc                       // the number of covariates
  init_ivector i_j(1,nu)            // the downstream release associated with each
                                    // upstream release
  init_vector Ru(1,nu)              // the size of each upstream release
  init_vector Rd(1,nd)              // the size of each downstream release
  init_matrix yu(1,nu,1,2)          // recoveries of upstream releases
  init_vector yo(1,nd)              // recoveries of downstream releases
  matrix tyu(1,2,1,nu)              // transpose of yu
  init_vector year88(1,nu)          // indicator for 1988
  init_matrix covariates(1,nu,1,nc) // the covariates
 !! tyu=trans(yu);
  number pi

PARAMETER_SECTION
  init_number spar0                             // coefficients for survival covariates
  init_vector spar1(2,nc,2)                     // coefficients for survival covariates
  init_bounded_vector pn(1,nu,1.e-6,1)          // net capture rates
  // init_vector betap(1,2)                        // params for capture rate of the net for each upstream release
  init_bounded_vector po(1,nd,1.e-6,1)          // ocean capture rate for each downstream release 
  init_bounded_vector phi(1,2,1,1000)           // overdispersion parameters
  vector s(1,nu)                                // survival rate for each upstream release
  vector scoff(1,nc);
  objective_function_value PL

INITIALIZATION_SECTION
  spar0 -0.1

PRELIMINARY_CALCS_SECTION
  pi           = 3.141592653589793;

PROCEDURE_SECTION

  scoff.initialize();
  scoff(1)=spar0;
  if (active(spar1)) scoff(2,nc)=spar1;

  dvar_vector pre_surv=mfexp(covariates*scoff);   

  // ------- Expected value calculations
  dvar_vector mu_s   = elem_div(pre_surv,    1.0+pre_surv);     // logistic link function
  // dvar_vector mu_s   = exp(-pre_surv);                             // cloglog link function

  // if p=f(covariates)
  //dvar_vector pre_p  = exp(betap(1)+betap(2)*year88);              
  //dvar_vector mu_p   = elem_div(pre_p,     1.0+pre_p);          // logistic link function
  // dvar_vector mu_p   = exp(-pre_p);                                // cloglog link function

  // if not ...
  dvar_vector mu_p = pn;

  dvar_vector mu_piu = po(i_j);
  dvar_vector mu_pid = po;
  dvar_vector mu_sp  = elem_prod(mu_s,mu_p);
  dvar_vector mu_spnpiu = elem_prod(elem_prod(mu_s,1-mu_p),mu_piu);

  // Expected recoveries
  dvar_vector muhat1  =  elem_prod(Ru,mu_sp);          // E[y_{ut}]
  dvar_vector muhat2  =  elem_prod(Ru,mu_spnpiu);      // E[y_{uo}]
  dvar_vector muhat3  =  elem_prod(Rd,mu_pid);         // E[y_{do}]

  // Residuals
  dvar_vector resid1 = tyu[1]-muhat1;
  dvar_vector resid2 = tyu[2]-muhat2;
  dvar_vector resid3 = yo-muhat3;

  // ------------- Constant overdispersion values, distinct per outcome ----------------------
  dvar_vector phiVmu1   = phi(1)*elem_prod(muhat1,1-mu_sp);
  dvar_vector phiVmu2   = phi(2)*elem_prod(muhat2,1-mu_spnpiu);
  dvar_vector phiVmu3   = elem_prod(muhat3,1-mu_pid);

  // *********************************************************************************************************//
  dvar_vector Pearson1 = elem_div(elem_prod(resid1,resid1),phiVmu1);
  dvar_vector Pearson2 = elem_div(elem_prod(resid2,resid2),phiVmu2);
  dvar_vector Pearson3 = elem_div(elem_prod(resid3,resid3),phiVmu3);

  PL  = sum(Pearson1+log(2.0*pi*phiVmu1)) + sum(Pearson2+log(2.0*pi*phiVmu2)) + 
        sum(Pearson3+log(2.0*pi*phiVmu3));
  PL  *= 0.5;
  PL  += 0.001*norm2(scoff);   // little penalty for stabilizing

