/* pair.c
   K. Newman, 
    Metropolis-Hastings implementation
    to estimate the posterior dist'n for hyperparameters of 
    hierarchical model for paired release-recovery data

    14 May: changed to single component moves of parameters
    23 May: logistic normals for S and p (when p is NOT unique)
    24 May: shock parameter for downstream releases
    25 May: sequential update of S (and p) and hyperparams
    19 September 2002: alternative trt of priors for logit-normal
    26 November 2002: modification of priors, "uniform" for S and p
     6-14 January 2003: runs with 3-fold xvalidation, modified proposals
*/

#include<stdio.h>
#include<math.h>
#include <string.h>
#include "../Recipes/nrutil.c"
#include "../Subroutines/header.h"  

 /* GLOBAL VARIABLES */
  FILE *fp1;                                     
 int numsims=0,burnin=10000;
 double **S;

 /* External functions */
  extern double dran1();
  extern double **dmatrix();
  extern void free_dmatrix();

main()
  {

 /* External functions */
  extern double min();
  extern double dgasdev();
  extern double lognormdens();
  extern double loglogitnormdens();
  extern double loglognormdens();
  extern double loglikelihood();

 /* Init file data */
 STRING upfile,downfile,Xfile,yrfile,outfile;


 long seed = -30;
 double *Ru, *Rd, *yut, *yuo, *ydo,**Xmat,*yrind;
 int    i,nu=61, nd=19, nc=12, numsims=100, j,k,l,popt, *uppair, *downpair,shockopt,data_opt;
 double *temp,temp1,temp2,
        alpha,num,den,
        prior_phiS, prior_phip,prior_pbeta0_mu,prior_Sbeta_SD,prior_pbeta_SD,
        priornew,priorold,fnew,fold,latentnew,latentold,qnew,qold,MHprob,
        *Sbeta_accept,*S_accept,pbeta_accept,*p_accept,*ocean_accept,shock_accept,
        **Sbeta,cand_S_coef,**Survival,*old_Survival,*new_Survival,*old_Xbeta,*new_Xbeta,
        cand_p1_coef,cand_p2_coef,**pbeta,**Capture,*old_Capture,*new_Capture,
        **thetapi, *old_pi,*new_pi,
        *shock,old_shock,new_shock,
        **fit_yut,**fit_yuo,**fit_ydo,
        *sigma2S_vec,old_sigma2S,new_sigma2S,sigma_phi,
        *sigma2p_vec,old_sigma2p,new_sigma2p,
        proposal_Sbeta_SD, proposal_Snoise_SD, 
        proposal_pbeta_SD, proposal_pnoise_SD,
        epsilon_S,epsilon_p,
        p_ub,pi_ub,shock_ub,delta_S,delta_p,delta_pi,delta_shock;

 /*------------------------------- User Input   --------------------------------*/
 printf("Enter #sims                                                             ");scanf("%d",&numsims);
 printf("Enter burn-in period                                                    ");scanf("%d",&burnin);
 printf("enter negative integer for seed                                         ");scanf("%d",&seed);
 printf("Choose p model: unique=1 f(cov)=2                                       ");scanf("%d",&popt);
 printf("\n");

/*--- setting the tuning parameters for the proposal distributions ---*/
 printf("Enter logit(S) Beta proposal param, proposal_Sbeta_SD (e.g. 0.5)        ");scanf("%lf",&proposal_Sbeta_SD);
 printf("Enter logit(S) noise VAR initial val            (e.g. 1.0  )            ");scanf("%lf",&old_sigma2S);
 printf("Enter logit(S)_noise VAR proposal param, proposal_Snoise_SD (e.g. 0.1)  ");scanf("%lf",&proposal_Snoise_SD);
 printf("Enter S proposal distn param (Unif) (e.g. 0.1)                          ");scanf("%lf",&delta_S);
 printf("\n");

 if(popt==2) {
  printf("Enter logit(p) Beta proposal param, proposal_pbetaSD (e.g. 0.1)         ");scanf("%lf",&proposal_pbeta_SD);
  printf("Enter logit(p) noise VAR initial val  (e.g. 0.5)                        ");scanf("%lf",&old_sigma2p);
  printf("Enter logit(p) noise VAR proposal param, proposal_pnoise_SD (e.g. 0.04) ");scanf("%lf",&proposal_pnoise_SD);
 } 
 printf("Enter p proposal distn param (Unif) (e.g. 0.0007)                       ");scanf("%lf",&delta_p);
 printf("\n");

 printf("Enter pi proposal dist (e.g. 0.002)                                    ");scanf("%lf",&delta_pi);
 printf("Enter 1 = shock 0 otherwise                                            ");scanf("%d",&shockopt);
 if(shockopt==1) {
  printf("Enter shock initial value (e.g. 0.9)                                   ");scanf("%lf",&old_shock);
  printf("Enter shock proposal distn param (Unif) (e.g. 0.005)                   ");scanf("%lf",&delta_shock);
 } else {
  old_shock=1.00;
 }

 printf("Enter data options: All=0, Train1=1, Train2=2, Train3=3                ");scanf("%d",&data_opt);
  
 if(data_opt == 0) {
       nu=61; upfile = "Up";        Xfile = "Xall";        yrfile = "Year"; outfile = "OUT";
 } else {
    if(data_opt == 1) {
       nu=43; upfile = "Up.xvalid1";Xfile = "Xall.xvalid1";yrfile = "Year.xvalid1"; outfile = "OUT.xvalid1"; 
    } else {
     if(data_opt ==2) { 
       nu=40; upfile = "Up.xvalid2";Xfile = "Xall.xvalid2";yrfile = "Year.xvalid2"; outfile = "OUT.xvalid2"; 
     } else {
       nu=40; upfile = "Up.xvalid3";Xfile = "Xall.xvalid3";yrfile = "Year.xvalid3"; outfile = "OUT.xvalid3";
      } 
    }
  }

  downfile  = "Down";

  uppair       = ivector(1,nu);
  downpair     = ivector(1,nd);
  Ru           = dvector(1,nu);
  Rd           = dvector(1,nd);
  yut          = dvector(1,nu);
  yuo          = dvector(1,nu);
  ydo          = dvector(1,nd);
  Xmat         = dmatrix(1,nu,1,nc);
  yrind        = dvector(1,nu);
  Sbeta_accept = dvector(1,nc);
  S_accept     = dvector(1,nu);
  p_accept     = dvector(1,nu);
  ocean_accept = dvector(1,nd);
 
  /*------------------------------- Data Loading --------------------------------*/
  if((fp1=fopen(upfile,"r")) == NULL) printf("Upstream file not open \n");
    for(i=1;i<=nu;i++) {
       fscanf(fp1,"%d %lf %lf %lf",&uppair[i],&Ru[i],&yut[i],&yuo[i]);
    }
    if(fclose(fp1) == EOF) printf("Upstream file not closing \n");
 
  if((fp1=fopen(downfile,"r")) == NULL) printf("Downstream file not open \n");
    for(i=1;i<=nd;i++) {
       fscanf(fp1,"%d %lf %lf",&downpair[i],&Rd[i],&ydo[i]);
    }
    if(fclose(fp1) == EOF) printf("Downstream file not closing \n");
 
  if((fp1=fopen(Xfile,"r")) == NULL) printf("Covariate file not open \n");
    for(i=1;i<=nu;i++) {
      for(j=1;j<=nc;j++) fscanf(fp1,"%lf",&Xmat[i][j]);
    }
    if(fclose(fp1) == EOF) printf("Covariate file not closing \n");
 
  if((fp1=fopen(yrfile,"r")) == NULL) printf("Year file not open \n");
    for(i=1;i<=nu;i++)  fscanf(fp1,"%lf",&yrind[i]);
    if(fclose(fp1) == EOF) printf("Year file not closing \n");
 

  /* ---- Space for simulated parameters and variables  ----*/
  sigma2S_vec  = dvector(1,numsims);
  Sbeta        = dmatrix(1,numsims,1,nc);
  old_Xbeta    = dvector(1,nu);
  new_Xbeta    = dvector(1,nu);
  Survival     = dmatrix(1,numsims,1,nu);
  old_Survival = dvector(1,nu);
  new_Survival = dvector(1,nu);
  shock = dvector(1,numsims);

  if(popt==2) {
     pbeta     = dmatrix(1,numsims,1,2);
     sigma2p_vec = dvector(1,numsims);
   }
  Capture      = dmatrix(1,numsims,1,nu);
  old_Capture  = dvector(1,nu);
  new_Capture  = dvector(1,nu);

  thetapi  = dmatrix(1,numsims,1,nd);
  old_pi   = dvector(1,nd); 
  new_pi   = dvector(1,nd); 

  fit_yut  = dmatrix(1,numsims,1,nu);
  fit_yuo  = dmatrix(1,numsims,1,nu);
  fit_ydo  = dmatrix(1,numsims,1,nd);

  /*------------------------------- Initializations   --------------------------------*/
  /*--- priors' hyperparameters for S and p, (pi is uniform) ----*/
  for(i=1;i <= nc; i++) Sbeta_accept[i]  = 0.0;
  for(i=1;i <= nu; i++) S_accept[i]      = 0.0;
  for(i=1;i <= nu; i++) p_accept[i]      = 0.0;
  for(i=1;i <= nd; i++) ocean_accept[i]  = 0.0;
  shock_accept  = 0.0;

  /*
  prior_Sbeta_SD = sqrt((9.869604/3.0)/13.0); 
  prior_phiS = 3.95; */   /* Exponential(3.95) prior for variance of S logit-normal error term */

  prior_Sbeta_SD = sqrt((9.869604/3.0-0.001)/12.0); 
  prior_phiS = 1000;   /* ..experimenting */

  /* prior for p, roughly U(0,0.1)
     3 components, each with 1/3 contribution to var(logit(p)) 
     Exponential(2.80) prior for variance of p logit-normal error term */
  /*
  prior_pbeta0_mu = -3.251;
  prior_pbeta_SD = sqrt(1.065/ 3.0);       
  prior_phip = 2.80;   
  */

  /* prior for p, roughly U(0,0.01)
     3 components, each with 1/3 contribution to var(logit(p)) 
     Exponential(3/0.987) prior for variance of p logit-normal error term */
  prior_pbeta0_mu = -5.60;
  /*
  prior_pbeta_SD = sqrt(0.987/ 3.0);
  prior_phip = 3/0.987; 
  */  
  prior_pbeta_SD = sqrt((0.987-0.001)/ 2.0);
  prior_phip = 1000;   /* ..experimenting */

  p_ub    = 0.01;   /* should adjust based on prior for p */
  pi_ub   = 0.08;
  shock_ub    = 0.999;
  shock[1]    =old_shock;
  new_shock   = shock[1];

 /* -------------- Initial Values ------------------------------------*/
 /* --------------Survival parameters ----------------- */
 for(j=1;j<=nc;j++) Sbeta[1][j] = 0.0 + dgasdev(&seed)*2.0;     

 Sbeta[1][1] =  1.0;
 sigma2S_vec[1] = old_sigma2S;
 for(k=1;k<=nu;k++) {
   old_Xbeta[k] = 0.0;
   for(j=1;j<=nc;j++) old_Xbeta[k] += Xmat[k][j]*Sbeta[1][j];
     epsilon_S = dgasdev(&seed)*sqrt(old_sigma2S);
     Survival[1][k] = exp(old_Xbeta[k]+epsilon_S)/(1.0+exp(old_Xbeta[k]+epsilon_S));
     old_Survival[k] = Survival[1][k];
   }

 /* --------------Capture parameters -------------------*/
 if(popt==2) {
   sigma2p_vec[1] = old_sigma2p;
   pbeta[1][1] = -6.0+dgasdev(&seed);
   pbeta[1][2] = +0.2+dgasdev(&seed);
   for(k=1;k<=nu;k++){
    epsilon_p = dgasdev(&seed)*sqrt(old_sigma2p);
    temp1   = exp(pbeta[1][1]+pbeta[1][2]*yrind[k] + epsilon_p);
    Capture[1][k] = temp1 / (1.0 + temp1);
    old_Capture[k] = Capture[1][k];
   }
  } else {
   for(k=1;k<=nu;k++) {
    Capture[1][k] = dran1(&seed)*p_ub;
    old_Capture[k] = Capture[1][k];
   }
 }

 /* ----------------Ocean recovery parameters ----------- */
 for(j=1;j<=nd;j++) thetapi[1][j] = (dran1(&seed))*pi_ub;
 for(j=1;j<=nd;j++) old_pi[j] = thetapi[1][j];

 fold = loglikelihood(Ru,yut,yuo,&nu,Rd,ydo,&nd,old_Survival,old_Capture,old_pi,uppair,shock[1]);
 printf("logL =%7.4lf \n",fold);

 for(k=1;k<=nu;k++) {
   fit_yut[1][k] = Ru[k]*old_Survival[k]*old_Capture[k];
   fit_yuo[1][k] = Ru[k]*old_Survival[k]*(1.0-old_Capture[k])*old_pi[uppair[k]];
  }
 for(k=1;k<=nd;k++) fit_ydo[1][k] = Rd[k]*old_shock*old_pi[k];


 /*---------------------------------------- Start of MCMC ----------------------------------------*/
 for(i=2;i<=numsims;i++) {
  /*----- Candidate Survival Parameters and Probabilities----- */

  /*initially all S parms are same as last time period */
  for(j=1;j<=nc;j++) Sbeta[i][j]    = Sbeta[i-1][j]; 

  for(j=1;j<=nc;j++) {
    /* update hyperparams first, then survival values */
    priorold = lognormdens(Sbeta[i-1][j],0.0,prior_Sbeta_SD);
    priorold += -prior_phiS*old_sigma2S;

    latentold = 0.0;
    for(k=1;k<=nu;k++) {
      old_Xbeta[k] = 0.0;
      for(l=1;l<=nc;l++) old_Xbeta[k] += Xmat[k][l]*Sbeta[i][l];
         temp1 = loglogitnormdens(old_Survival[k],old_Xbeta[k],sqrt(old_sigma2S));
         latentold += temp1;
      }

    /* Generating candidate beta_S* and sigma2S*:  Metropolis type sampler */    
    cand_S_coef = Sbeta[i-1][j]+dgasdev(&seed)*proposal_Sbeta_SD; 
    new_sigma2S = old_sigma2S*exp(dgasdev(&seed)*proposal_Snoise_SD);

    priornew    = lognormdens(cand_S_coef,0.0,prior_Sbeta_SD);
    priornew   += -prior_phiS*new_sigma2S;

    qnew     = lognormdens(cand_S_coef,Sbeta[i-1][j],proposal_Sbeta_SD);
    qold     = lognormdens(Sbeta[i-1][j],cand_S_coef,proposal_Sbeta_SD);

    qnew    += loglognormdens(new_sigma2S,log(old_sigma2S),proposal_Snoise_SD);
    qold    += loglognormdens(old_sigma2S,log(new_sigma2S),proposal_Snoise_SD);

    Sbeta[i][j] = cand_S_coef;  /* possibly a temporary assignment */
    latentnew = 0.0;
    for(k=1;k<=nu;k++) {
       new_Xbeta[k] = 0.0;
       for(l=1;l<=nc;l++) new_Xbeta[k] += Xmat[k][l]*Sbeta[i][l];
          temp1 = loglogitnormdens(old_Survival[k],new_Xbeta[k],sqrt(new_sigma2S));
          latentnew += temp1;
       }

    num   = priornew+latentnew+qold;
    den   = priorold+latentold+qnew;
    alpha = min(1.0,exp(num-den));

    MHprob = dran1(&seed);
    if(MHprob <= alpha)  {
     sigma2S_vec[i]    = new_sigma2S;
     old_sigma2S       = new_sigma2S;
     Sbeta[i][j] = cand_S_coef;
     Sbeta_accept[j] += 1;
    } else {
       sigma2S_vec[i]    = old_sigma2S;
       Sbeta[i][j] = Sbeta[i-1][j];
    }

    /*----now updating survival values ----*/
   for(k=1;k<=nu;k++) {
    old_Xbeta[k] = 0.0;
    for(l=1;l<=nc;l++) old_Xbeta[k] += Xmat[k][l]*Sbeta[i][l];

    /* proposal: uniform "centered" around previous value */
    if(old_Survival[k] >= delta_S && (old_Survival[k] <= 1.0 -delta_S)) {
      new_Survival[k] = dran1(&seed)*2.0*delta_S - delta_S + old_Survival[k];
    } else
    {
     temp1 = dran1(&seed)*2.0*delta_S;
     if(old_Survival[k] > delta_S) {
       new_Survival[k] = 1.0 - temp1;
     }
     else new_Survival[k] = temp1;
    }

    old_Xbeta[k] = 0.0;
    for(l=1;l<=nc;l++) old_Xbeta[k] += Xmat[k][l]*Sbeta[i][l];
    latentold = loglogitnormdens(old_Survival[k],old_Xbeta[k],sqrt(old_sigma2S));
    latentnew = loglogitnormdens(new_Survival[k],old_Xbeta[k],sqrt(old_sigma2S));

    /* proposal is the latent density (logit normal), thence latent dens and proposal cancel */
    /*
    epsilon_S       = dgasdev(&seed)*sqrt(old_sigma2S);
    new_Survival[k] = exp(old_Xbeta[k]+epsilon_S)/(1+exp(old_Xbeta[k]+epsilon_S));
    latentold = 0.0; latentnew = 0.0;
    */

    fold = yut[k]*log(old_Survival[k]*old_Capture[k]) +
             yuo[k]*log(old_Survival[k]*(1.0-old_Capture[k])*old_pi[uppair[k]])+
             (Ru[k]-yut[k]-yuo[k])*log(1.0-old_Survival[k]*old_Capture[k]-
             old_Survival[k]*(1.0-old_Capture[k])*old_pi[uppair[k]]);

    fnew = yut[k]*log(new_Survival[k]*old_Capture[k]) +
             yuo[k]*log(new_Survival[k]*(1.0-old_Capture[k])*old_pi[uppair[k]])+
             (Ru[k]-yut[k]-yuo[k])*log(1.0-new_Survival[k]*old_Capture[k]-
             new_Survival[k]*(1.0-old_Capture[k])*old_pi[uppair[k]]);

    num   = fnew+latentnew;
    den   = fold+latentold;
    alpha = min(1.0,exp(num-den));

    MHprob = dran1(&seed);
      if(MHprob <= alpha) {
        old_Survival[k] = new_Survival[k];
        Survival[i][k]  = new_Survival[k];
        S_accept[k]    += 1.0;
      } else Survival[i][k]  = old_Survival[k];
    }
  } /* end of loop thru Survival parameters */


  /*-----------------------  Candidate Capture parameters and Probabilities-------------- */
   if(popt==1) {
     /* ----Release unique capture probabilities -------------- */
     /*  proposals are on Uniform interval of length 2*delta_p */
     for(j=1;j<=nu;j++) {
       temp1 = dran1(&seed)*2.0*delta_p;
       if(old_Capture[j] >= delta_p && (old_Capture[j] <= p_ub -delta_p)) {
          new_Capture[j] = temp1 - delta_p + old_Capture[j];
       } else {
         if(old_Capture[j] > delta_p) {
           new_Capture[j] =  p_ub - temp1;
         }
         else new_Capture[j] = temp1;
       }

       fnew = yut[j]*log(old_Survival[j]*new_Capture[j])+
              yuo[j]*log(old_Survival[j]*(1.0-new_Capture[j])*old_pi[uppair[j]]) +
              (Ru[j]-yut[j]-yuo[j])*log(1.0-old_Survival[j]*new_Capture[j]-
              old_Survival[j]*(1-new_Capture[j])*old_pi[uppair[j]]);

       fold = yut[j]*log(old_Survival[j]*old_Capture[j])+
              yuo[j]*log(old_Survival[j]*(1.0-old_Capture[j])*old_pi[uppair[j]]) +
              (Ru[j]-yut[j]-yuo[j])*log(1.0-old_Survival[j]*old_Capture[j]-
              old_Survival[j]*(1-old_Capture[j])*old_pi[uppair[j]]);

       num = fnew;
       den = fold;
       alpha = min(1.0,exp(num-den));

       MHprob = dran1(&seed);
       if(MHprob <= alpha) {            
        Capture[i][j]  = new_Capture[j];
        old_Capture[j] = new_Capture[j];
        p_accept[j] += 1.0;
      } else { 
       Capture[i][j] = old_Capture[j];
       }
     }
   } else {  /* covariate based p */
      /* update hyperparams first, then capture values */
      priorold = lognormdens(pbeta[i-1][1],prior_pbeta0_mu,prior_pbeta_SD) +
                 lognormdens(pbeta[i-1][2],0.0,prior_pbeta_SD);
      priorold += -prior_phip*old_sigma2p;

      latentold = 0.0;
      for(k=1;k<=nu;k++) {
         temp1   = pbeta[i-1][1]+pbeta[i-1][2]*yrind[k];
         temp2   = loglogitnormdens(old_Capture[k],temp1,sqrt(old_sigma2p));
         latentold += temp2;
      }

      /* Generating candidate beta_p* and sigma2p*:  Metropolis type sampler */    
      cand_p1_coef = pbeta[i-1][1]+dgasdev(&seed)*proposal_pbeta_SD; 
      cand_p2_coef = pbeta[i-1][2]+dgasdev(&seed)*proposal_pbeta_SD; 
      new_sigma2p  = old_sigma2p*exp(dgasdev(&seed)*proposal_pnoise_SD);
      priornew     = lognormdens(cand_p1_coef,prior_pbeta0_mu,prior_pbeta_SD) +
                     lognormdens(cand_p2_coef,0.0,prior_pbeta_SD);
      priornew += -prior_phip*new_sigma2p;

     /* some assignments "unnecessary", due to cancellations */
     qnew     = lognormdens(cand_p1_coef,pbeta[i-1][1],proposal_pbeta_SD)+
                lognormdens(cand_p2_coef,pbeta[i-1][2],proposal_pbeta_SD);
     qnew    += loglognormdens(new_sigma2p,log(old_sigma2p),proposal_pnoise_SD);

     qold     = lognormdens(pbeta[i-1][1],cand_p1_coef,proposal_pbeta_SD) +
                lognormdens(pbeta[i-1][2],cand_p2_coef,proposal_pbeta_SD);
     qold    += loglognormdens(old_sigma2p,log(new_sigma2p),proposal_pnoise_SD);

     latentnew = 0.0;
     for(k=1;k<=nu;k++) {
        temp1      = cand_p1_coef+cand_p2_coef*yrind[k];
        temp2      = loglogitnormdens(old_Capture[k],temp1,sqrt(new_sigma2p));
        latentnew += temp2;
     }

     num   = priornew+latentnew+qold;
     den   = priorold+latentold+qnew;
     alpha = min(1.0,exp(num-den));

     MHprob = dran1(&seed);
     if(MHprob <= alpha) {
       sigma2p_vec[i] = new_sigma2p;
       old_sigma2p    = new_sigma2p;
       pbeta[i][1]    = cand_p1_coef;
       pbeta[i][2]    = cand_p2_coef;
       pbeta_accept  += 1;
     }
     else {
       sigma2p_vec[i] = old_sigma2p;
       pbeta[i][1]    = pbeta[i-1][1];
       pbeta[i][2]    = pbeta[i-1][2];
     }

     /*----now updating Capture values ----*/
     for(k=1;k<=nu;k++) {
        /* proposal: uniform "centered" around previous value */
        if(old_Capture[k] >= delta_p && (old_Capture[k] <= p_ub -delta_p)) {
           new_Capture[k] = dran1(&seed)*2.0*delta_p - delta_p + old_Capture[k];
        } else
	{
         temp1 = dran1(&seed)*2.0*delta_p;
         if(old_Capture[k] > delta_p) {
           new_Capture[k] = p_ub - temp1;
         }
           else new_Capture[k] = temp1;
         }

        temp1     = pbeta[i][1]+pbeta[i][2]*yrind[k];
        latentold = loglogitnormdens(old_Capture[k],temp1,sqrt(old_sigma2p));
        latentnew = loglogitnormdens(new_Capture[k],temp1,sqrt(old_sigma2p));

        fold = yut[k]*log(old_Survival[k]*old_Capture[k]) +
               yuo[k]*log(old_Survival[k]*(1.0-old_Capture[k])*old_pi[uppair[k]])+
               (Ru[k]-yut[k]-yuo[k])*log(1.0-old_Survival[k]*old_Capture[k]-
               old_Survival[k]*(1.0-old_Capture[k])*old_pi[uppair[k]]);

        fnew = yut[k]*log(old_Survival[k]*new_Capture[k]) +
               yuo[k]*log(old_Survival[k]*(1.0-new_Capture[k])*old_pi[uppair[k]])+
               (Ru[k]-yut[k]-yuo[k])*log(1.0-old_Survival[k]*new_Capture[k]-
               old_Survival[k]*(1.0-new_Capture[k])*old_pi[uppair[k]]);

        num   = fnew+latentnew;
        den   = fold+latentold;
        alpha = min(1.0,exp(num-den));

        MHprob = dran1(&seed);
        if(MHprob <= alpha) {
          old_Capture[k] = new_Capture[k];
          Capture[i][k]  = new_Capture[k];
          p_accept[k] += 1.0;
         }
         else Capture[i][k]  = old_Capture[k];
      }
   }


  /*----- Ocean recovery parameters-------------------------- */
  /* update shock by itself, else no movement of some pi's */
   if(shockopt==1) {
     old_shock = shock[i-1];
     if(old_shock >= delta_shock && (old_shock <= shock_ub - delta_shock)) {
        new_shock = dran1(&seed)*2.0*delta_shock - delta_shock + old_shock;
       } else {
         temp1 = dran1(&seed)*2.0*delta_shock;
         if(old_shock > delta_shock) {
          new_shock = shock_ub - temp1;
         }
         else new_shock = temp1;
        }
       }

    fnew = 0.0;
    fold = 0.0;
    for(j=1;j<=nd;j++) {
     fnew += ydo[j]*log(new_shock*old_pi[j])+(Rd[j]-ydo[j])*log(1.0-new_shock*old_pi[j]);
     fold += ydo[j]*log(old_shock*old_pi[j])+(Rd[j]-ydo[j])*log(1.0-old_shock*old_pi[j]);
    }
    alpha = min(1.0,exp(fnew-fold));

   MHprob = dran1(&seed);
   if(MHprob <= alpha) {
     shock[i]  = new_shock;
     old_shock = new_shock;
     shock_accept += 1;
    } else { 
     shock[i]  = old_shock;
  }

   /* ------------------ update ocean pi one at a time  --------------------------- */
   for(j=1;j<=nd;j++) {
     if(old_pi[j] >= delta_pi && (old_pi[j] <= pi_ub -delta_pi)) {
        new_pi[j] = dran1(&seed)*2.0*delta_pi - delta_pi + old_pi[j];
     } else {
       temp1 = dran1(&seed)*2.0*delta_pi;
       if(old_pi[j] > delta_pi) {
                new_pi[j] = pi_ub - temp1;
          }
        else new_pi[j] = temp1;
      }

     fnew = 0.0;
     fold = 0.0;
     for(k=1;k<=nu;k++) {
      if(uppair[k]==j) {
        fnew += yuo[k]*log(Survival[i][k]*(1.0-Capture[i][k])*new_pi[j]) +
        (Ru[k]-yut[k]-yuo[k])*log(1.0-Survival[i][k]*Capture[i][k]-Survival[i][k]*(1-Capture[i][k])*new_pi[j]);

        fold += yuo[k]*log(Survival[i][k]*(1.0-Capture[i][k])*old_pi[j]) +
        (Ru[k]-yut[k]-yuo[k])*log(1.0-Survival[i][k]*Capture[i][k]-Survival[i][k]*(1-Capture[i][k])*old_pi[j]);
      }
    }
   fnew += ydo[j]*log(shock[i]*new_pi[j])+(Rd[j]-ydo[j])*log(1.0-shock[i]*new_pi[j]);
   fold += ydo[j]*log(shock[i]*old_pi[j])+(Rd[j]-ydo[j])*log(1.0-shock[i]*old_pi[j]);
   alpha = min(1.0,exp(fnew-fold));

   MHprob = dran1(&seed);
   if(MHprob <= alpha) {
      old_pi[j] = new_pi[j];
      thetapi[i][j] = new_pi[j];
      ocean_accept[j] += 1.0;
    } else { 
      thetapi[i][j]  = old_pi[j];
    }

   }

  /* ------------------Fitted Values ------------------------------------------ */
  for(k=1;k<=nu;k++) {
      fit_yut[i][k] = Ru[k]*Survival[i][k]*Capture[i][k];
      fit_yuo[i][k] = Ru[k]*Survival[i][k]*(1.0-Capture[i][k])*old_pi[uppair[k]];
  }
  for(k=1;k<=nd;k++) fit_ydo[i][k] = Rd[k]*shock[i]*old_pi[k];

  fold = loglikelihood(Ru,yut,yuo,&nu,Rd,ydo,&nd,old_Survival,old_Capture,old_pi,uppair,shock[i]);
  if(popt==1) {
        printf("Iter=%3d fold=%10.0f SigS=%8.7f shock=%3.2f \n",i,fold,sigma2S_vec[i],shock[i]);
  } else {
    printf("Iter=%3d fold=%10.0f SigS=%8.7f SigP=%8.7f shock=%3.2f \n",i,fold,sigma2S_vec[i],sigma2p_vec[i],shock[i]);
  }
 }

/*--------------------------------------- End of MCMC   ---------------------------------------*/

   /* ------------------output -------------------- */

  /* ---Output the simulated parameters: betas' for S, for p (if relevant)  -------------------- */
  if(popt==1) {
   if((fp1=fopen(outfile,"w")) == NULL) printf("output file not open \n");
       for(i=burnin+1;i<=numsims;i++) {
 	   fprintf(fp1,"%6.5f ",sigma2S_vec[i]);
 	   for(j=1;j<=nc;j++) fprintf(fp1,"%6.5f ",Sbeta[i][j]);
            if(popt==2) for(j=1;j<=2 ;j++) fprintf(fp1,"%6.5f ",pbeta[i][j]);
 	   fprintf(fp1,"\n");
 	 }
     if(fclose(fp1) == EOF) printf("output file not closing \n");
  } else {
  if((fp1=fopen(outfile,"w")) == NULL) printf("output file not open \n");
      for(i=burnin+1;i<=numsims;i++) {
	   fprintf(fp1,"%6.5f %6.5f ",sigma2S_vec[i],sigma2p_vec[i]);
	   for(j=1;j<=nc;j++) fprintf(fp1,"%6.5f ",Sbeta[i][j]);
           if(popt==2) for(j=1;j<=2 ;j++) fprintf(fp1,"%6.5f ",pbeta[i][j]);
	   fprintf(fp1,"\n");
	 }
    if(fclose(fp1) == EOF) printf("output file not closing \n");
  }

  outfile = "OUT.shock";
  if((fp1=fopen(outfile,"w")) == NULL) printf("output file for shock not open \n");
      for(i=burnin+1;i<=numsims;i++) 
	   fprintf(fp1,"%4.3lf \n",shock[i]);
  if(fclose(fp1) == EOF) printf("output file for shock not closing \n");

  /* ---Output the fitted recoveries, yut, yuo, ydo  -------------------- */
  outfile = "OUT.yut";
  if((fp1=fopen(outfile,"w")) == NULL) printf("output file for yut not open \n");
      for(i=burnin+1;i<=numsims;i++) {
	   for(k=1;k<=nu;k++) fprintf(fp1,"%4.0lf ",fit_yut[i][k]);fprintf(fp1,"\n");
	 }
    if(fclose(fp1) == EOF) printf("output file for yut not closing \n");

  outfile = "OUT.yuo";
  if((fp1=fopen(outfile,"w")) == NULL) printf("output file for yuo not open \n");
      for(i=burnin+1;i<=numsims;i++) {
	   for(k=1;k<=nu;k++) fprintf(fp1,"%4.0lf ",fit_yuo[i][k]);fprintf(fp1,"\n");
	 }
  if(fclose(fp1) == EOF) printf("output file for yuo not closing \n");

  outfile = "OUT.ydo";
  if((fp1=fopen(outfile,"w")) == NULL) printf("output file for ydo not open \n");
      for(i=burnin+1;i<=numsims;i++) {
	   for(k=1;k<=nd;k++) fprintf(fp1,"%4.0lf ",fit_ydo[i][k]);fprintf(fp1,"\n");
	 }
  if(fclose(fp1) == EOF) printf("output file for ydo not closing \n");

  /* ---Output the Survival, Capture, and Ocean Recovery Parameters --- */
  outfile = "OUT.S";
  if((fp1=fopen(outfile,"w")) == NULL) printf("output file for S not open \n");
      for(i=burnin+1;i<=numsims;i++) {
	   for(k=1;k<=nu;k++) fprintf(fp1,"%4.3f ",Survival[i][k]);fprintf(fp1,"\n");
	 }
  if(fclose(fp1) == EOF) printf("output file for S not closing \n");

  outfile = "OUT.p";
  if((fp1=fopen(outfile,"w")) == NULL) printf("output file for p not open \n");
      for(i=burnin+1;i<=numsims;i++) {
	   for(k=1;k<=nu;k++) fprintf(fp1,"%7.6f ",Capture[i][k]);fprintf(fp1,"\n");
	 }
  if(fclose(fp1) == EOF) printf("output file for p not closing \n");

  outfile = "OUT.pi";
  if((fp1=fopen(outfile,"w")) == NULL) printf("output file for pi not open \n");
      for(i=burnin+1;i<=numsims;i++) {
	   for(k=1;k<=nd;k++) fprintf(fp1,"%7.6f ",thetapi[i][k]);fprintf(fp1,"\n");
	 }
  if(fclose(fp1) == EOF) printf("output file for pi not closing \n");

   printf("Sbeta Acceptance \n");
   for(i=1;i<= nc; i++) printf("%5.3f \n",Sbeta_accept[i]/numsims);
   printf("S Acceptance     \n");
   for(i=1;i<= nu; i++) printf("%5.3f ",S_accept[i]/(numsims*nc)); printf("\n");
   if(popt==2) {
    printf("Theta p Acceptance= %5.3f \n",pbeta_accept/numsims);
   }
   printf("p  Acceptance     \n");
   for(i=1;i<= nu; i++) printf("%5.3f ",p_accept[i]/numsims); printf("\n");
   printf("pi Acceptance     \n");
   for(i=1;i<= nd; i++) printf("%5.3f ",ocean_accept[i]/numsims); printf("\n");
   if(shockopt==1)
     printf("shock Acceptance     = %5.3f \n",shock_accept/(1.0*numsims));

}
 /* ---------------------------------------------------------- */
 /* END OF MAIN PROGRAM */
 /* ---------------------------------------------------------- */
 
 /* ---------------------------------------------------------- */
 double min(a,b) 
   double a,b;
 {
  if(a<b) return(a);
  return(b);
 }

 /* ---------------------------------------------------------- */
 double loglikelihood(Ru,yut,yuo,nu,Rd,ydo,nd,S,p,pi,uppair,shock)
 int *nu,*nd;
 MAT_1 Ru,Rd,yut,yuo,ydo,S,p,pi;
 INT_1 uppair;
 double shock; {

  int i,j;
  double temp, upcomp=0.0, downcomp=0.0, loglike=0.0;

  for(i=1;i<=*nu;i++) {
     temp = yut[i]*log(S[i]*p[i]) + yuo[i]*log(S[i]*(1.0-p[i])*pi[uppair[i]]) +
            (Ru[i]-yut[i]-yuo[i])*log(1.0-S[i]*p[i]-S[i]*(1.0-p[i])*pi[uppair[i]]);
     upcomp += temp;
  }
  
  for(i=1;i<=*nd;i++) {
     temp = ydo[i]*log((shock)*pi[i])+ (Rd[i]-ydo[i])*log(1.0-(shock)*pi[i]);
     downcomp += temp;
  }
  loglike = upcomp+downcomp;
  return(loglike);
}
  
 /* ---------------------------------------------------------- */
 double lognormdens(y,mu,sigma)
   double y,mu,sigma; {

  double temp;
  double pi=3.14159265358979;
 
  temp = (y-mu)/sigma;
  temp = -0.5*(log(2.0*pi*sigma*sigma)+temp*temp);
  return(temp);
} 
 /* ---------------------------------------------------------- */
 double loglogitnormdens(y,mu,sigma)
   double y,mu,sigma; {

  double temp;
  double pi=3.14159265358979;
 
  temp = (log(y/(1-y))-mu)/sigma;
  temp = -0.5*(log(2.0*pi*sigma*sigma)+temp*temp)-log(y*(1-y));
  return(temp);
} 
 /* ---------------------------------------------------------- */
 double loglognormdens(y,mu,sigma)
   double mu,sigma,y; {

  double temp;
  double pi=3.14159265358979;
 
  temp = (log(y)-mu)/sigma;
  temp = -0.5*log(2*pi*sigma*sigma)-0.5*temp*temp-log(y);
  return(temp);
} 
